import react from 'react';
import { useAtom } from 'jotai';

import { TileKeyObject } from './types';

import { coordinateSystemAtom } from './Map';
import { range } from 'lodash';
import tileUri from './uris';
import TiledLayer from './TiledLayer';

export interface LayerStackProperties {
  /**
   * A key identifying the top left tile
   */
  keyObject: TileKeyObject;
  /**
   * Number of {@link components/map/TiledLayer!TiledLayer}.
   */
  numberOfTiledLayers?: number;
}

/**
 *
 * @param props
 * @returns A stack of layers embedded in an SVG element
 *
 * This component does the conversion between the {@link components/map/Map!CoordinateSystem} stored
 * in the {@link components/map/Map!coordinateSystemAtom} atom and the {@link components/map/TiledLayer!TiledLayer}
 * components which units are in tiles.
 *
 */
export const LayerStack: react.FC<LayerStackProperties> = (
  props: LayerStackProperties
) => {
  const [coordinateSystem] = useAtom(coordinateSystemAtom);

  const numberOfTiledLayers =
    props.numberOfTiledLayers === undefined ? 1 : props.numberOfTiledLayers;

  const activeTiledLayer = Math.floor(numberOfTiledLayers / 2);

  const getTiledLayer = (i: number) => {
    const relativeZoomLevel = i - activeTiledLayer;
    const zoom = 2 ** relativeZoomLevel;
    const origin = {
      x: props.keyObject.x * zoom,
      y: props.keyObject.y * zoom,
    };
    const keyObject = {
      provider: props.keyObject.provider,
      zoomLevel: props.keyObject.zoomLevel + relativeZoomLevel,
      x: Math.floor(origin.x),
      y: Math.floor(origin.y),
    };
    const shift = {
      x: -origin.x,
      y: -origin.y,
    };
    const key = tileUri({
      provider: keyObject.provider,
      zoomLevel: keyObject.zoomLevel,
    });

    return (
      <TiledLayer
        key={key}
        keyObject={keyObject}
        shift={shift}
        zoom={zoom * 256}
      />
    );
  };

  //  console.log(`tiledLayers: ${JSON.stringify(tiledLayers)}`);

  return (
    <svg
      width={window.innerWidth}
      height={window.innerHeight}
      data-testid='layer-stack'
    >
      <g
        transform={`translate(${coordinateSystem.shift.x}, ${coordinateSystem.shift.y}) scale(${coordinateSystem.zoom})`}
      >
        {
          // Tiled layers with less detail
          range(0, activeTiledLayer).map(getTiledLayer)
        }
        {
          // Tiled layers with more details
          range(numberOfTiledLayers - 1, activeTiledLayer, -1).map(
            getTiledLayer
          )
        }
        {
          // And the active one
          getTiledLayer(activeTiledLayer)
        }
      </g>
    </svg>
  );
};

export default LayerStack;
